"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const withoutAliasSymbols = (typescript, type, cb) => {
    const aliasSymbols = new Map();
    // Remove alias symbols from type arguments of reference types.
    // Example: Array<MyUnionType>
    if (type.flags & typescript.TypeFlags.Object) {
        const otype = type;
        if (otype.objectFlags & typescript.ObjectFlags.Reference) {
            const rtype = type;
            if (rtype.typeArguments) {
                for (const arg of rtype.typeArguments) {
                    if (arg.aliasSymbol) {
                        aliasSymbols.set(arg, arg.aliasSymbol);
                        arg.aliasSymbol = undefined;
                    }
                }
            }
        }
    }
    try {
        return cb();
    }
    finally {
        for (const [type, aliasSymbol] of aliasSymbols.entries()) {
            type.aliasSymbol = aliasSymbol;
        }
    }
};
class TypeChecker {
    constructor(typescript, virtualServiceHost, getTypeChecker) {
        this.typescript = typescript;
        this.virtualServiceHost = virtualServiceHost;
        this.getTypeChecker = getTypeChecker;
        this.registry = this.typescript.createDocumentRegistry(true);
        this.service = this.typescript.createLanguageService(virtualServiceHost, this.registry, false);
    }
    getType(node) {
        const checker = this.getTypeChecker();
        const type = checker.getTypeAtLocation(node);
        return withoutAliasSymbols(this.typescript, type, () => checker.typeToString(type, undefined, this.typescript.TypeFormatFlags.NoTruncation |
            this.typescript.TypeFormatFlags.InTypeAlias));
    }
    check(content) {
        const fileName = `_${Math.floor(Math.random() * 100000)}.ts`;
        return this.virtualServiceHost.withFile(fileName, content, () => this.service.getSemanticDiagnostics(fileName));
    }
}
exports.TypeChecker = TypeChecker;
//# sourceMappingURL=type-checker.js.map